@@ -5,11 +5,7 @@ import (
 	"fmt"
 	"os"
 
-	"github.com/prometheus/client_golang/prometheus"
-
 	// kube
-	oauthv1 "github.com/openshift/api/oauth/v1"
-	routev1 "github.com/openshift/api/route/v1"
 	appsv1 "k8s.io/api/apps/v1"
 	corev1 "k8s.io/api/core/v1"
 	apierrors "k8s.io/apimachinery/pkg/api/errors"
@@ -18,8 +14,10 @@ import (
 
 	// openshift
 	configv1 "github.com/openshift/api/config/v1"
+	oauthv1 "github.com/openshift/api/oauth/v1"
 	operatorsv1 "github.com/openshift/api/operator/v1"
 	operatorv1 "github.com/openshift/api/operator/v1"
+	routev1 "github.com/openshift/api/route/v1"
 	"github.com/openshift/console-operator/pkg/api"
 	"github.com/openshift/console-operator/pkg/console/subresource/util"
 	"github.com/openshift/console-operator/pkg/crypto"
@@ -28,28 +26,17 @@ import (
 	"github.com/openshift/library-go/pkg/operator/resourcesynccontroller"
 
 	// operator
+
 	customerrors "github.com/openshift/console-operator/pkg/console/errors"
+	"github.com/openshift/console-operator/pkg/console/metrics"
+	"github.com/openshift/console-operator/pkg/console/status"
 	configmapsub "github.com/openshift/console-operator/pkg/console/subresource/configmap"
 	deploymentsub "github.com/openshift/console-operator/pkg/console/subresource/deployment"
 	oauthsub "github.com/openshift/console-operator/pkg/console/subresource/oauthclient"
 	routesub "github.com/openshift/console-operator/pkg/console/subresource/route"
 	secretsub "github.com/openshift/console-operator/pkg/console/subresource/secret"
-	servicesub "github.com/openshift/console-operator/pkg/console/subresource/service"
 )
 
-var (
-	// metric: console_url{url="https://<url>"} 1
-	consoleURLMetric = prometheus.NewGaugeVec(prometheus.GaugeOpts{
-		Name: "console_url",
-		Help: "URL of the console exposed on the cluster",
-		// one label
-	}, []string{"url"})
-)
-
-func init() {
-	prometheus.MustRegister(consoleURLMetric)
-}
-
 // The sync loop starts from zero and works its way through the requirements for a running console.
 // If at any point something is missing, it creates/updates that piece and immediately dies.
 // The next loop will pick up where they previous left off and move the process forward one step.
@@ -63,35 +50,28 @@ func (co *consoleOperator) sync_v400(updatedOperatorConfig *operatorv1.Console,
 
 	rt, rtChanged, rtErrReason, rtErr := co.SyncRoute(set.Operator)
 	toUpdate = toUpdate || rtChanged
-	co.HandleProgressingOrDegraded(updatedOperatorConfig, "RouteSync", rtErrReason, rtErr)
+	status.HandleProgressingOrDegraded(updatedOperatorConfig, "RouteSync", rtErrReason, rtErr)
 	if rtErr != nil {
 		return rtErr
 	}
 
-	svc, svcChanged, svcErrReason, svcErr := co.SyncService(set.Operator)
-	toUpdate = toUpdate || svcChanged
-	co.HandleProgressingOrDegraded(updatedOperatorConfig, "ServiceSync", svcErrReason, svcErr)
-	if svcErr != nil {
-		return svcErr
-	}
-
 	cm, cmChanged, cmErrReason, cmErr := co.SyncConfigMap(set.Operator, set.Console, set.Infrastructure, rt)
 	toUpdate = toUpdate || cmChanged
-	co.HandleProgressingOrDegraded(updatedOperatorConfig, "ConfigMapSync", cmErrReason, cmErr)
+	status.HandleProgressingOrDegraded(updatedOperatorConfig, "ConfigMapSync", cmErrReason, cmErr)
 	if cmErr != nil {
 		return cmErr
 	}
 
 	serviceCAConfigMap, serviceCAChanged, serviceCAErrReason, serviceCAErr := co.SyncServiceCAConfigMap(set.Operator)
 	toUpdate = toUpdate || serviceCAChanged
-	co.HandleProgressingOrDegraded(updatedOperatorConfig, "ServiceCASync", serviceCAErrReason, serviceCAErr)
+	status.HandleProgressingOrDegraded(updatedOperatorConfig, "ServiceCASync", serviceCAErrReason, serviceCAErr)
 	if serviceCAErr != nil {
 		return serviceCAErr
 	}
 
 	trustedCAConfigMap, trustedCAConfigMapChanged, trustedCAErrReason, trustedCAErr := co.SyncTrustedCAConfigMap(set.Operator)
 	toUpdate = toUpdate || trustedCAConfigMapChanged
-	co.HandleProgressingOrDegraded(updatedOperatorConfig, "TrustedCASync", trustedCAErrReason, trustedCAErr)
+	status.HandleProgressingOrDegraded(updatedOperatorConfig, "TrustedCASync", trustedCAErrReason, trustedCAErr)
 	if trustedCAErr != nil {
 		return trustedCAErr
 	}
@@ -100,25 +80,25 @@ func (co *consoleOperator) sync_v400(updatedOperatorConfig *operatorv1.Console,
 	customLogoCanMount, customLogoErrReason, customLogoError := co.SyncCustomLogoConfigMap(updatedOperatorConfig)
 	// If the custom logo sync fails for any reason, we are degraded, not progressing.
 	// The sync loop may not settle, we are unable to honor it in current state.
-	co.HandleProgressingOrDegraded(updatedOperatorConfig, "CustomLogoSync", customLogoErrReason, customLogoError)
+	status.HandleProgressingOrDegraded(updatedOperatorConfig, "CustomLogoSync", customLogoErrReason, customLogoError)
 
 	sec, secChanged, secErr := co.SyncSecret(set.Operator)
 	toUpdate = toUpdate || secChanged
-	co.HandleProgressingOrDegraded(updatedOperatorConfig, "OAuthClientSecretSync", "FailedApply", secErr)
+	status.HandleProgressingOrDegraded(updatedOperatorConfig, "OAuthClientSecretSync", "FailedApply", secErr)
 	if secErr != nil {
 		return secErr
 	}
 
 	oauthClient, oauthChanged, oauthErrReason, oauthErr := co.SyncOAuthClient(set.Operator, sec, rt)
 	toUpdate = toUpdate || oauthChanged
-	co.HandleProgressingOrDegraded(updatedOperatorConfig, "OAuthClientSync", oauthErrReason, oauthErr)
+	status.HandleProgressingOrDegraded(updatedOperatorConfig, "OAuthClientSync", oauthErrReason, oauthErr)
 	if oauthErr != nil {
 		return oauthErr
 	}
 
 	actualDeployment, depChanged, depErrReason, depErr := co.SyncDeployment(set.Operator, cm, serviceCAConfigMap, trustedCAConfigMap, sec, rt, set.Proxy, customLogoCanMount)
 	toUpdate = toUpdate || depChanged
-	co.HandleProgressingOrDegraded(updatedOperatorConfig, "DeploymentSync", depErrReason, depErr)
+	status.HandleProgressingOrDegraded(updatedOperatorConfig, "DeploymentSync", depErrReason, depErr)
 	if depErr != nil {
 		return depErr
 	}
@@ -130,7 +110,7 @@ func (co *consoleOperator) sync_v400(updatedOperatorConfig *operatorv1.Console,
 	klog.V(4).Infof("sync loop 4.0.0 resources updated: %v", toUpdate)
 	klog.V(4).Infoln("-----------------------")
 
-	co.HandleProgressing(updatedOperatorConfig, "SyncLoopRefresh", "InProgress", func() error {
+	status.HandleProgressing(updatedOperatorConfig, "SyncLoopRefresh", "InProgress", func() error {
 		if toUpdate {
 			return errors.New("Changes made during sync updates, additional sync expected.")
 		}
@@ -144,7 +124,7 @@ func (co *consoleOperator) sync_v400(updatedOperatorConfig *operatorv1.Console,
 		return nil
 	}())
 
-	co.HandleAvailable(func() (conf *operatorv1.Console, prefix string, reason string, err error) {
+	status.HandleAvailable(func() (conf *operatorv1.Console, prefix string, reason string, err error) {
 		prefix = "Deployment"
 		if !deploymentsub.IsReady(actualDeployment) {
 			return updatedOperatorConfig, prefix, "InsufficientReplicas", errors.New(fmt.Sprintf("%v pods available for console deployment", actualDeployment.Status.ReadyReplicas))
@@ -155,7 +135,7 @@ func (co *consoleOperator) sync_v400(updatedOperatorConfig *operatorv1.Console,
 		return updatedOperatorConfig, prefix, "", nil
 	}())
 
-	co.HandleAvailable(updatedOperatorConfig, "Route", "FailedAdmittedIngress", func() error {
+	status.HandleAvailable(updatedOperatorConfig, "Route", "FailedAdmittedIngress", func() error {
 		if !routesub.IsAdmitted(rt) {
 			return errors.New("console route is not admitted")
 		}
@@ -185,9 +165,7 @@ func (co *consoleOperator) sync_v400(updatedOperatorConfig *operatorv1.Console,
 
 	defer func() {
 		klog.V(4).Infof("sync loop 4.0.0 complete")
-		if svcChanged {
-			klog.V(4).Infof("\t service changed: %v", svc.GetResourceVersion())
-		}
+
 		if rtChanged {
 			klog.V(4).Infof("\t route changed: %v", rt.GetResourceVersion())
 		}
@@ -212,22 +190,13 @@ func (co *consoleOperator) sync_v400(updatedOperatorConfig *operatorv1.Console,
 }
 
 func (co *consoleOperator) SyncConsoleConfig(consoleConfig *configv1.Console, consoleURL string) (*configv1.Console, error) {
+	oldURL := consoleConfig.Status.ConsoleURL
 	updated := consoleConfig.DeepCopy()
-
-	// track the URL state in prometheus before we update it
-	if consoleConfig.Status.ConsoleURL != consoleURL {
-		// not using this URL anymore
-		consoleURLMetric.WithLabelValues(consoleConfig.Status.ConsoleURL).Set(0)
-	}
-	if len(consoleURL) != 0 {
-		// only update to new if we have a url
-		consoleURLMetric.WithLabelValues(consoleURL).Set(1)
-	}
-
 	if updated.Status.ConsoleURL != consoleURL {
 		klog.V(4).Infof("updating console.config.openshift.io with url: %v", consoleURL)
 		updated.Status.ConsoleURL = consoleURL
 	}
+	metrics.HandleConsoleURL(oldURL, consoleURL)
 	return co.consoleConfigClient.UpdateStatus(updated)
 }
 
@@ -309,14 +278,14 @@ func (co *consoleOperator) SyncConfigMap(
 	operatorConfig *operatorv1.Console,
 	consoleConfig *configv1.Console,
 	infrastructureConfig *configv1.Infrastructure,
-	rt *routev1.Route) (consoleConfigMap *corev1.ConfigMap, changed bool, reason string, err error) {
+	consoleRoute *routev1.Route) (consoleConfigMap *corev1.ConfigMap, changed bool, reason string, err error) {
 
 	managedConfig, mcErr := co.configMapClient.ConfigMaps(api.OpenShiftConfigManagedNamespace).Get(api.OpenShiftConsoleConfigMapName, metav1.GetOptions{})
 	if mcErr != nil && !apierrors.IsNotFound(mcErr) {
 		return nil, false, "FailedManagedConfig", mcErr
 	}
 
-	defaultConfigmap, _, err := configmapsub.DefaultConfigMap(operatorConfig, consoleConfig, managedConfig, infrastructureConfig, rt)
+	defaultConfigmap, _, err := configmapsub.DefaultConfigMap(operatorConfig, consoleConfig, managedConfig, infrastructureConfig, consoleRoute)
 	if err != nil {
 		return nil, false, "FailedConsoleConfigBuilder", err
 	}
@@ -395,16 +364,6 @@ func (co *consoleOperator) SyncTrustedCAConfigMap(operatorConfig *operatorv1.Con
 	return actual, true, "", err
 }
 
-// apply service
-// there is nothing special about our service, so no additional error handling is needed here.
-func (co *consoleOperator) SyncService(operatorConfig *operatorv1.Console) (consoleService *corev1.Service, changed bool, reason string, err error) {
-	svc, svcChanged, svcErr := resourceapply.ApplyService(co.serviceClient, co.recorder, servicesub.DefaultService(operatorConfig))
-	if svcErr != nil {
-		return nil, false, "FailedApply", svcErr
-	}
-	return svc, svcChanged, "", svcErr
-}
-
 // apply route
 // - be sure to test that we don't trigger an infinite loop by stomping on the
 //   default host name set by the server, or any other values. The ApplyRoute()
